package ac.hiu.kt.testdb;

import org.springframework.core.io.*;
import org.springframework.dao.*;
import org.springframework.jdbc.core.*;
import org.springframework.jdbc.core.support.*;
import org.springframework.jdbc.object.*;
import java.sql.*;
import java.util.*;
import javax.sql.*;

public class SampleBookJdbcDaoImple extends JdbcDaoSupport {
  private FindBookQuery findBookQuery = null;
  private FindAllBookQuery findAllBookQuery = null;
  private InsertBookSql insertBookSql = null;
  private UpdateBookSql updateBookSql = null;
  private DeleteBookSql deleteBookSql = null;

  // JdbcSupportDaõ\bhI[oCh
  protected void initDao() {
    findBookQuery = new FindBookQuery(getDataSource());
    findAllBookQuery = new FindAllBookQuery(getDataSource());
    insertBookSql = new InsertBookSql(getDataSource());
    updateBookSql = new UpdateBookSql(getDataSource());
    deleteBookSql = new DeleteBookSql(getDataSource());
  }

  public List findBook(String userId)  throws DataAccessException {
    return findBookQuery.execute(userId);
  }

  public List findBook()  throws DataAccessException {
    return findAllBookQuery.execute();
  }

  public int insertBook(String id, String name) 
                                         throws DataAccessException {
    String[] param = {id, name};
    return insertBookSql.update(param);
  }

  public int updateBook(String id, String name) 
                                         throws DataAccessException {
    String[] param = {name, id};
    return updateBookSql.update(param);
  }

  public int deleteBook(String id) throws DataAccessException {
    return deleteBookSql.update(id);
  }
  // RDBMSIy[VYNXpNX̒`
  //   SampleBooǩ(BaseNXj
  private static class FindBookQueryBase extends MappingSqlQuery {
    public FindBookQueryBase(DataSource ds, String sql) {
      super(ds,sql);
    }

    public Object mapRow(ResultSet rs, int rowNumber) 
                                         throws SQLException {
      String name = rs.getString("name");
      String id = rs.getString("id");
      Book book = new Book(id, name);
      return book;
    }
  }

  //   SampleBooǩ(idw肵āj
  private static class FindBookQuery extends FindBookQueryBase {
    private static final String SQL_QUERY
      = "select id, name from SampleBook where id=?;";

    public FindBookQuery(DataSource ds) {
      super(ds,SQL_QUERY);
      super.declareParameter(new SqlParameter("id", Types.VARCHAR));
      compile();
    }
  }

  //   SampleBooǩ(Sj
  private static class FindAllBookQuery extends FindBookQueryBase {
    private static final String SQL_QUERY
      = "select id, name from SampleBook;";

    public FindAllBookQuery(DataSource ds) {
      super(ds,SQL_QUERY);
      compile();
    }
  }

  //   SampleBook̒ǉ
  private static class InsertBookSql extends SqlUpdate {
    private static final String SQL_QUERY
      = "insert into SampleBook values (?, ?);";

    public InsertBookSql(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("id", Types.VARCHAR));
      super.declareParameter(new SqlParameter("name", Types.VARCHAR));
      compile();
    }
  }

  //   SampleBook̍XV
  private static class UpdateBookSql extends SqlUpdate {
    private static final String SQL_QUERY
      = "update SampleBook set name=? where id=?;";

    public UpdateBookSql(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("name", Types.VARCHAR));
      super.declareParameter(new SqlParameter("id", Types.VARCHAR));
      compile();
    }
  }

  //   SampleBook̍폜
  private static class DeleteBookSql extends SqlUpdate {
    private static final String SQL_QUERY
      = "delete from SampleBook where id=?;";

    public DeleteBookSql(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("id", Types.VARCHAR));
      compile();
    }
  }
}
