package ac.hiu.kt.bgame.dao;

import java.sql.*;
import java.util.*;
import javax.sql.*;
import org.springframework.jdbc.core.*;
import org.springframework.jdbc.core.support.*;
import org.springframework.jdbc.object.*;
import ac.hiu.kt.bgame.model.*;

public class GameJdbcDaoImple extends JdbcDaoSupport 
                              implements GameDao {
  private FindFinishedGameQuery findFinishedGameQuery = null;
  private InsertGameSql insertGameSql = null;
  private UpdateGameSql updateGameSql = null;
  private FindGameQueryByTeamNo findGameQueryByTeamNo=null;

  protected void initDao() {
    findFinishedGameQuery = new FindFinishedGameQuery(getDataSource());
    insertGameSql = new InsertGameSql(getDataSource());
    updateGameSql = new UpdateGameSql(getDataSource());
    findGameQueryByTeamNo = new FindGameQueryByTeamNo(getDataSource());
  }

  public List findFinishedGame(int taikaiId) {
    return findFinishedGameQuery.execute(taikaiId);
  }

  public Game findGameByTeamNo(int taikaiId, int team1No, int team2No) {
    Object[] params = new Object[3];
    params[0] = new Integer(taikaiId);
    params[1] = new Integer(team1No);
    params[2] = new Integer(team2No);
    List results=findGameQueryByTeamNo.execute(params);
    if (results.size()!=1) {
      return null;
    } else {
      return (Game)results.get(0);
    }
  }
  
  public boolean insertGame(int taikaiId, Game game) {
    int team1No = game.getTeam1No();
    int team2No = game.getTeam2No();
    int team1Point=game.getTeam1Point();
    int team2Point=game.getTeam2Point();
    Object[] params = new Object[5];
    params[0] = new Integer(taikaiId);
    params[1] = new Integer(team1No);
    params[2] = new Integer(team2No);
    params[3] = new Integer(team1Point);
    params[4] = new Integer(team2Point);
    int n = insertGameSql.update(params);
    if (n!=1) {
      return false;
    } else {
      return true;
    }
  }

  public boolean updateGame(int taikaiId, Game game) {
    int team1No = game.getTeam1No();
    int team2No = game.getTeam2No();
    int team1Point = game.getTeam1Point();
    int team2Point = game.getTeam2Point();
    Object[] params = new Object[5];
    params[0] = new Integer(team1Point);
    params[1] = new Integer(team2Point);
    params[2] = new Integer(taikaiId);
    params[3] = new Integer(team1No);
    params[4] = new Integer(team2No);
    int n = updateGameSql.update(params);
    //System.out.println("team1No:"+team1No+" team1Point:"+team1Point);
    //System.out.println("team2No:"+team2No+" team2Point:"+team2Point);
    //System.out.println("n:"+n);
    if (n!=1) {
      return false;
    } else {
      return true;
    }
  }

  // ID̏IQ[wvNX
  private class FindFinishedGameQuery extends MappingSqlQuery {
    private static final String SQL_QUERY =
        "select team1No, team2No, team1Point, team2Point from game "+
        "where taikaiId=? and team1Point>=0;";
    public FindFinishedGameQuery(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("id", Types.INTEGER));
      compile();
    }

    public Object mapRow(ResultSet rs, int row) throws SQLException {
      int team1No=rs.getInt("team1No");
      int team2No=rs.getInt("team2No");
      int team1Point=rs.getInt("team1Point");
      int team2Point=rs.getInt("team2Point");
      Game game = new Game(team1No, team2No, "", "" );
      game.setPoints(team1Point, team2Point);
      return game;
    }
  }

  // w肵`[̃Q[wvNX
  private class FindGameQueryByTeamNo extends MappingSqlQuery {
    private static final String SQL_QUERY =
        "select team1No, team2No, team1Point, team2Point from game "+
        "where taikaiId=? and team1No=? and team2No=?;";
    public FindGameQueryByTeamNo(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("id", Types.INTEGER));
      super.declareParameter(new SqlParameter("team1No", Types.INTEGER));
      super.declareParameter(new SqlParameter("team2No", Types.INTEGER));
      compile();
    }

    public Object mapRow(ResultSet rs, int row) throws SQLException {
      int team1No=rs.getInt("team1No");
      int team2No=rs.getInt("team2No");
      int team1Point=rs.getInt("team1Point");
      int team2Point=rs.getInt("team2Point");
      Game game = new Game(team1No, team2No, "", "" );
      game.setPoints(team1Point, team2Point);
      return game;
    }
  }

  // Q[o^pwvNX
  private class InsertGameSql extends SqlUpdate {
    private static final String SQL_INSERT =
        "insert into game values(?,?,?,?,?);";
    public InsertGameSql(DataSource ds) {
      super(ds, SQL_INSERT);
      super.declareParameter(new SqlParameter("taikaiId", Types.INTEGER));
      super.declareParameter(new SqlParameter("team1No", Types.INTEGER));
      super.declareParameter(new SqlParameter("team2No", Types.INTEGER));
      super.declareParameter(new SqlParameter("team1Point", Types.INTEGER));
      super.declareParameter(new SqlParameter("team2Point", Types.INTEGER));
      compile();
    }
  }

  // Q[XVpwvNX
  private class UpdateGameSql extends SqlUpdate {
    private static final String SQL_UPDATE =
        "update game set team1Point=?, team2Point=? "+
        "where taikaiId=? and team1No=? and team2No=?;";
    public UpdateGameSql(DataSource ds) {
      super(ds, SQL_UPDATE);
      super.declareParameter(new SqlParameter("team1Point", Types.INTEGER));
      super.declareParameter(new SqlParameter("team2Point", Types.INTEGER));
      super.declareParameter(new SqlParameter("taikaiId", Types.INTEGER));
      super.declareParameter(new SqlParameter("team1No", Types.INTEGER));
      super.declareParameter(new SqlParameter("team2No", Types.INTEGER));
      compile();
    }
  }
}
