package ac.hiu.kt.bgame.dao;

import java.sql.*;
import java.util.*;
import javax.sql.*;
import org.springframework.jdbc.core.*;
import org.springframework.jdbc.core.support.*;
import org.springframework.jdbc.object.*;

import ac.hiu.kt.bgame.model.*;

public class TeamJdbcDaoImple extends JdbcDaoSupport
                              implements TeamDao {

  private FindTeamQuery findTeamQuery=null;
  private FindTeamQueryByNo findTeamQueryByNo = null;
  private InsertTeamSql insertTeamSql = null;
      
  protected void initDao() {
    findTeamQuery = new FindTeamQuery(getDataSource());
    findTeamQueryByNo = new FindTeamQueryByNo(getDataSource());
    insertTeamSql = new InsertTeamSql(getDataSource());
  }

  public List findTeam(int taikaiId) {
    Object[] params = {new Integer(taikaiId)} ;
    List results=findTeamQuery.execute(params);
    return results;
  }
      
  public String findTeamName(int taikaiId, int teamNo) {
    Object[] params = {new Integer(taikaiId), new Integer(teamNo)} ;
    List results=findTeamQueryByNo.execute(params);
    if (results.size()!=1) {
      return null;
    } else {
      return (String)results.get(0);
    }
  }

  public boolean insertTeam(int taikaiId, int no, String name) {
    Object[] params = new Object[3];
    params[0]=new Integer(taikaiId);
    params[1]=new Integer(no);
    params[2]=name;
    int n = insertTeamSql.update(params);
    if (n!=1) {
      return false;
    } else {
      return true;
    }
  }

  // `[ԍ`[wvNX
  private class FindTeamQueryByNo extends MappingSqlQuery {
    private static final String SQL_QUERY =
        "select name from team where taikaiId=? and teamNo=?;";
    public FindTeamQueryByNo(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("taikaiId", Types.INTEGER));
      super.declareParameter(new SqlParameter("teamNo", Types.INTEGER));
      compile();
    }
    
    public Object mapRow(ResultSet rs, int row) throws SQLException {
      String name=rs.getString("name");
      return name;
    }
  }
  
  // id̃`[wvNX
  private class FindTeamQuery extends MappingSqlQuery {
    private static final String SQL_QUERY =
        "select taikaiId, teamno, name from team where taikaiId=?;";
    public FindTeamQuery(DataSource ds) {
      super(ds, SQL_QUERY);
      super.declareParameter(new SqlParameter("taikaiId", Types.INTEGER));
      compile();
    }
    
    public Object mapRow(ResultSet rs, int row) throws SQLException {
      int no=rs.getInt("teamno");
      String name=rs.getString("name");
      Team team=new Team(no, name);
      return team;
    }
  }

  // `[o^pwvNX
  private class InsertTeamSql extends SqlUpdate {
    private static final String SQL_INSERT =
        "insert into team values(?, ?, ?);";
    public InsertTeamSql(DataSource ds) {
      super(ds, SQL_INSERT);
      super.declareParameter(new SqlParameter("id", Types.INTEGER));
      super.declareParameter(new SqlParameter("no", Types.INTEGER));
      super.declareParameter(new SqlParameter("name", Types.VARCHAR));
      compile();
    }
  }
}
